import pandas as pd
from xpctl.utils import write_config_file
import json
import ast

pd.set_option('display.expand_frame_repr', False)
pd.set_option("display.max_rows", None)


def pack_result(results):
    """ List of results to event data"""
    metrics = set([r.metric for r in results])
    d = {metric: [] for metric in metrics}
    for result in results:
        d[result.metric].append(result.value)
    return pd.DataFrame(d)


def pack_aggregate_result(results, aggregate_fns):
    metrics = [r.metric for r in results]
    metrics = set(metrics)
    d = {metric: {} for metric in metrics}
    for result in results:
        for value in result.values:
            aggregate_fn = value.aggregate_fn
            if aggregate_fn not in aggregate_fns:
                continue
            score = value.score
            if aggregate_fn in d[result.metric]:
                d[result.metric][aggregate_fn].append(score)
            else:
                d[result.metric][aggregate_fn] = [score]

    dfs = {metric: pd.DataFrame.from_dict(d[metric])[list(aggregate_fns)] for metric in metrics}
    return pd.concat(dfs.values(), axis=1, keys=dfs.keys())


def get_prop_value(exp, prop_name):
    return exp.__dict__.get('_'+prop_name)


def insert_in_df(prop_name_loc, df, exp):
    for prop_name, location in prop_name_loc.items():
        df.insert(location, column=prop_name, value=[get_prop_value(exp, prop_name)]*len(df))
        
        
def experiment_to_df(exp, prop_name_loc={}, event_type='test_events', sort=None):
    prop_name_loc = {'sha1': 0, 'id': 1, 'username':  2} if not prop_name_loc else prop_name_loc
    if event_type == 'train_events' and exp.train_events:
        result_df = pack_result(exp.train_events)
        insert_in_df(prop_name_loc, result_df, exp)
    if event_type == 'valid_events' and exp.valid_events:
        result_df = pack_result(exp.valid_events)
        insert_in_df(prop_name_loc, result_df, exp)
    if event_type == 'test_events' and exp.test_events:
        result_df = pack_result(exp.test_events)
        insert_in_df(prop_name_loc, result_df, exp)
    if sort is not None:
        result_df.sort_values(by=sort, inplace=True)
    return result_df


def experiment_aggregate_to_df(exp_agg, prop_name_loc, event_type='test_events',
                               aggregate_fns=['min', 'max', 'avg', 'std']):
    event_dfs = []
    if event_type == 'train_events':
        train_df = pack_aggregate_result(exp_agg.train_events, aggregate_fns)
        insert_in_df(prop_name_loc, train_df, exp_agg)
        event_dfs.append(train_df)
    if event_type == 'valid_events':
        valid_df = pack_aggregate_result(exp_agg.valid_events, aggregate_fns)
        insert_in_df(prop_name_loc, valid_df, exp_agg)
        event_dfs.append(valid_df)
    if event_type == 'test_events':
        test_df = pack_aggregate_result(exp_agg.test_events, aggregate_fns)
        insert_in_df(prop_name_loc, test_df, exp_agg)
        event_dfs.append(test_df)
    result_df = pd.DataFrame()
    for event_df in event_dfs:
        result_df = result_df.append(event_df)
    return result_df


def experiment_aggregate_list_to_df(exp_aggs, event_type='test_events',
                                    aggregate_fns=['min', 'max', 'avg', 'std']):
    result_df = pd.DataFrame()
    prop_name_loc = {'dataset': 0, 'sha1': 1, 'num_exps': 2}
    for exp_agg in exp_aggs:
        result_df = result_df.append(experiment_aggregate_to_df(exp_agg, prop_name_loc, event_type, aggregate_fns))
    return result_df


def experiment_list_to_df(exps, prop_name_loc={}, event_type='test_events'):
    result_df = pd.DataFrame()
    prop_name_loc = {'sha1': 0, 'id': 1, 'username':  2} if not prop_name_loc else prop_name_loc
    for exp in exps:
        result_df = result_df.append(experiment_to_df(exp, prop_name_loc, event_type, sort=None))
    return result_df


def write_to_config_file(config_obj, filename):
    write_config_file(config_obj, filename)


def task_summary_to_df(tasksummary):
    def identity(x): return x
    summary = tasksummary.summary
    all_results = []
    for dataset in summary:
        for user, num_exps in summary[dataset]:
            all_results.append([user, dataset, num_exps])
    return pd.DataFrame(all_results, columns=['user', 'dataset', 'num_exps']).groupby(['user', 'dataset'])\
        .agg([identity]).rename(columns={'identity': ''})


def task_summaries_to_df(tasksummaries):
    def identity(x): return x
    all_results = []
    for tasksummary in tasksummaries:
        task = tasksummary.task
        summary = tasksummary.summary
        for dataset in summary:
            for user, num_exps in summary[dataset]:
                all_results.append([task, user, dataset, num_exps])
    return pd.DataFrame(all_results, columns=['task', 'user', 'dataset', 'num_exps']).groupby(['task', 'user', 'dataset'])\
        .agg([identity]).rename(columns={'identity': ''})


def read_config_stream(config):
    try:
        return json.loads(config)
    except json.decoder.JSONDecodeError:
        try:
            return ast.literal_eval(config)
        except ValueError:
            return None



# coding: utf-8

# flake8: noqa
from __future__ import absolute_import
# import models into model package
from xpctl.xpserver.models.aggregate_result import AggregateResult
from xpctl.xpserver.models.aggregate_result_values import AggregateResultValues
from xpctl.xpserver.models.experiment import Experiment
from xpctl.xpserver.models.experiment_aggregate import ExperimentAggregate
from xpctl.xpserver.models.response import Response
from xpctl.xpserver.models.result import Result
from xpctl.xpserver.models.task_summary import TaskSummary



import sys
import getpass
import json
import os

from click_shell import shell
import click
from xpctl.xpclient import Configuration
from xpctl.xpclient.api import XpctlApi
from xpctl.xpclient import ApiClient
from xpctl.xpclient.rest import ApiException
from xpctl.clihelpers import experiment_to_df, experiment_aggregate_list_to_df, experiment_list_to_df, \
    task_summary_to_df, task_summaries_to_df, read_config_stream
from xpctl.utils import to_swagger_experiment, store_model, write_config_file
from mead.utils import hash_config, get_dataset_from_key, index_by_label
from baseline.utils import read_config_file

EVENT_TYPES = {
    "train": "train_events", "Train": "train_events",
    "test": "test_events", "Test": "test_events",
    "valid": "valid_events", "Valid": "valid_events",
    "dev": "valid_events", "Dev": "valid_events"
}


class ServerManager(object):
    
    api = None
    host = None
    
    @staticmethod
    def get():
        if ServerManager.api is None:
            config = Configuration(host=ServerManager.host)
            api_client = ApiClient(config)
            ServerManager.api = XpctlApi(api_client)
            
        if ServerManager.api is not None:
            click.echo(click.style(
                "connection with xpctl server successful with [host]: {}".format(ServerManager.host), fg='green'
            ))
            return ServerManager.api
        click.echo(click.style("server connection unsuccessful, aborting", fg='red'))
        sys.exit(1)


@shell(prompt="xpctl > ", intro="Starting xpctl...")
@click.option('--host', help="server host", default=None)
@click.option('--config', help="xpctl config", default='~/xpctlcred.json')
def cli(host, config):
    if host is not None:
        ServerManager.host = host
    else:
        ServerManager.host = read_config_file(os.path.expanduser(config))['host']


@cli.command()
@click.argument('task')
@click.argument('eid')
def getmodelloc(task, eid):
    """Get the model location for a particular task and record id"""
    ServerManager.get()
    result = ServerManager.api.get_model_location(task, eid)
    if result.response_type == 'success':
        click.echo(click.style(result.message, fg='green'))
    else:
        click.echo(click.style(result.message, fg='red'))


@cli.command()
@click.option('--event_type', default='test', help="train/ dev/ test")
@click.option('--output', help='output file (csv)', default=None)
@click.option('--metric', multiple=True, help="list of metrics (prec, recall, f1, accuracy),[multiple]: "
                                              "--metric f1 --metric acc", default=None)
@click.option('--sort', help="specify one metric to sort the results", default=None)
@click.option('--output_fields', multiple=True, help="which field(s) you want to see in output",
              default=['username', 'sha1'])
@click.argument('task')
@click.argument('eid')
def experiment(task, eid, event_type, output, metric, sort, output_fields):
    """Get the details for an experiment"""
    event_type = EVENT_TYPES[event_type]
    ServerManager.get()
    try:
        result = ServerManager.api.experiment_details(task, eid, event_type=event_type, metric=metric)
        prop_name_loc = {k: i for i, k in enumerate(output_fields)}
        result_df = experiment_to_df(exp=result, prop_name_loc=prop_name_loc, event_type=event_type, sort=sort)
        if output is None:
            click.echo(result_df)
        else:
            result_df.to_csv(output)
    except ApiException as e:
        click.echo(click.style(json.loads(e.body)['detail'], fg='red'))


@cli.command()
@click.option('--metric', multiple=True, help="list of metrics (prec, recall, f1, accuracy),[multiple]: --metric f1 "
                                              "--metric acc")
@click.option('--sort', help="specify one metric to sort the results")
@click.option('--nconfig', help='number of experiments to aggregate', type=int, default=1)
@click.option('--event_type', default='test', help="train/ dev/ test")
@click.option('--n', help='number of rows to show', type=int, default=-1)
@click.option('--output', help='output file (csv)', default=None)
@click.option('--aggregate_fn', help='aggregate functions', multiple=True,
              type=click.Choice(['min', 'max', 'avg', 'std']), default=['avg', 'std'])
@click.option('--label', type=str, help='filter results with a certain label')
@click.argument('task')
@click.argument('dataset')
def results(task, dataset, metric, sort, nconfig, event_type, n, output, aggregate_fn, label):
    event_type = EVENT_TYPES[event_type]
    reduction_dim = 'sha1'
    ServerManager.get()
    try:
        if label:
            result = ServerManager.api.get_results_by_prop(task, dataset=dataset, label=label,
                                                           reduction_dim=reduction_dim, metric=metric, sort=sort,
                                                           numexp_reduction_dim=nconfig, event_type=event_type)
        else:
            result = ServerManager.api.get_results_by_prop(task, dataset=dataset,
                                                           reduction_dim=reduction_dim, metric=metric, sort=sort,
                                                           numexp_reduction_dim=nconfig, event_type=event_type)
        result_df = experiment_aggregate_list_to_df(exp_aggs=result, event_type=event_type, aggregate_fns=aggregate_fn)
        if n != -1:
            result_df = result_df.head(n)
        if output is None:
            click.echo(result_df)
        else:
            result_df.to_csv(output)
    except ApiException as e:
        click.echo(click.style(json.loads(e.body)['detail'], fg='red'))


@cli.command()
@click.option('--user', multiple=True, help="list of users (testuser, root), [multiple]: --user a --user b")
@click.option('--metric', multiple=True, help="list of metrics (prec, recall, f1, accuracy),[multiple]: --metric f1 "
                                              "--metric acc")
@click.option('--sort', help="specify one metric to sort the results", default=None)
@click.option('--event_type', default='test', help="specify one metric to sort the results")
@click.option('--n', help='number of experiments', type=int)
@click.option('--output', help='output file (csv)')
@click.option('--output_fields', multiple=True, help="which field(s) you want to see in output",
              default=['username', 'eid'])
@click.argument('task')
@click.argument('sha1')
def details(task, sha1, user, metric, sort, event_type, n, output, output_fields):
    """
    Shows the results for all experiments for a particular config (sha1). Optionally filter out by user(s), metric(s),
    label or sort by one metric. Shows the results on the test data by default, provide event_type (train/valid/test)
    to see for other events.
    """
    event_type = EVENT_TYPES[event_type]
    ServerManager.get()
    try:
        result = ServerManager.api.list_experiments_by_prop(task, sha1=sha1, user=user, metric=metric,
                                                            sort=sort, event_type=event_type)
        prop_name_loc = {k: i for i, k in enumerate(output_fields)}
        result_df = experiment_list_to_df(exps=result, prop_name_loc=prop_name_loc, event_type=event_type)
        if n != -1:
            result_df = result_df.head(n)
        if output is None:
            click.echo(result_df)
        else:
            result_df.to_csv(output)
    except ApiException as e:
        click.echo(click.style(json.loads(e.body)['detail'], fg='red'))


@cli.command()
@click.argument('task')
@click.argument('sha1')
@click.argument('filename')
def config2json(task, sha1, filename):
    """Exports the config file for an experiment as a json file."""
    ServerManager.get()
    try:
        result = ServerManager.api.config2json(task, sha1)
        write_config_file(result, filename)
    except ApiException as e:
        click.echo(click.style(json.loads(e.body)['detail'], fg='red'))


@cli.command()
@click.option('--task')
def lbsummary(task):
    """
    Provides a summary of the leaderboard. Options: taskname. If you provide
    a taskname, it will show all users and datasets for that task.
    This is helpful because we often forget what datasets were
    used for a task, which are the necessary parameters for the commands
    `results` and `best` and `tasksummary`. Shows the summary for all available tasks if no option is specified.
    """
    ServerManager.get()
    try:
        if task is not None:
            result = task_summary_to_df(ServerManager.api.task_summary(task))
        else:
            result = task_summaries_to_df(ServerManager.api.summary())
        click.echo(result)
    except ApiException as e:
        click.echo(click.style(json.loads(e.body)['detail'], fg='red'))


@cli.command()
@click.argument('task')
@click.argument('eid')
@click.argument('label')
def updatelabel(task, label, eid):
    """Update the _label_ for an experiment (identified by its eid) for a task"""
    ServerManager.get()
    result = ServerManager.api.update_property(task, eid, prop='label', value=label)
    if result.response_type == 'success':
        click.echo(click.style(result.message, fg='green'))
    else:
        click.echo(click.style(result.message, fg='red'))


@cli.command()
@click.argument('task')
@click.argument('eid')
def delete(task, eid):
    """Deletes a record from the database and the associated model file from model-checkpoints if it exists."""
    ServerManager.get()
    result = ServerManager.api.remove_experiment(task, eid)
    if result.response_type == 'success':
        click.echo(click.style(result.message, fg='green'))
    else:
        click.echo(click.style(result.message, fg='red'))


@cli.command()
@click.option("--user", help="username", default=getpass.getuser())
@click.option("--cbase", help="path to the base structure for the model checkpoint files:"
                              "such as ../tagger/tagger-model-tf-11967 or /home/ds/tagger/tagger-model-tf-11967")
@click.option("--cstore", default="/data/model-checkpoints", help="location of the model checkpoint store")
@click.option("--label", default=None, help="label for the experiment")
@click.argument('task')
@click.argument('config')
@click.argument('log')
@click.argument('dataset')
def putresult(task, config, log, dataset, user, label, cbase, cstore):
    """
    Puts the results in a database. provide task name, config file, the reporting log file, and the dataset index file
    used in the experiment. Optionally can put the model files in a persistent storage.
    """
    logf = log.format(task)
    if not os.path.exists(logf):
        click.echo(click.style("the log file at {} doesn't exist, provide a valid location".format(logf), fg='red'))
        return
    if not os.path.exists(config):
        click.echo(click.style("the config file at {} doesn't exist, provide a valid location".format(config), fg='red'))
        return
    if not os.path.exists(dataset):
        click.echo(click.style("the dataset file at {} doesn't exist, provide a valid location".format(dataset), fg='red'))
        return
    config_obj = read_config_file(config)
    datasets_set = index_by_label(read_config_file(dataset))
    dataset_key = config_obj['dataset']
    dataset_key = get_dataset_from_key(dataset_key, datasets_set)
    config_obj['dataset'] = dataset_key['label']
    ServerManager.get()
    result = ServerManager.api.put_result(task, to_swagger_experiment(task, config_obj, log, username=user, label=label))
    if result.response_type == 'success':
        eid = result.message
        click.echo(click.style('results stored with experiment: {}'.format(result.message), fg='green'))
        if cbase is None:
            return
        result = store_model(checkpoint_base=cbase, config_sha1=hash_config(read_config_file(config)),
                             checkpoint_store=cstore, print_fn=click.echo)
        if result is not None:
            click.echo(click.style('model stored at {}'.format(result), fg='green'))
            update_result = ServerManager.api.update_property(task, eid, prop='checkpoint', value=result)
            if update_result.response_type == 'success':
                click.echo(click.style(update_result.message, fg='green'))
            else:
                click.echo(click.style(update_result.message, fg='red'))
        else:
            click.echo(click.style('failed to store model'.format(result), fg='red'))
    else:
        click.echo(click.style(result.message, fg='red'))
        

@cli.command()
@click.option("--cstore", default="/data/model-checkpoints", help="location of the model checkpoint store")
@click.argument('task')
@click.argument('eid')
@click.argument('cbase')
def putmodel(task, eid, cbase, cstore):
    """
    Puts the baseline model files in persistent storage. provide task, id and model base (""tagger-model-tf-").
    optionally provide the storage location (/data/model-checkpoints by default)
    """
    ServerManager.get()
    event_type = EVENT_TYPES['test']
    metric = []
    try:
        result = ServerManager.api.experiment_details(task, eid, event_type=event_type, metric=metric)
        config_obj = read_config_stream(result.config)
        if config_obj is None:
            click.echo('can not process the config for experiment {} in {} database'.format(task, eid))
            sys.exit(1)
        result = store_model(checkpoint_base=cbase, config_sha1=hash_config(config_obj),
                             checkpoint_store=cstore, print_fn=click.echo)
        if result is not None:
            click.echo(click.style('model stored at {}'.format(result), fg='green'))
            update_result = ServerManager.api.update_property(task, eid, prop='checkpoint', value=result)
            if update_result.response_type == 'success':
                click.echo(click.style(update_result.message, fg='green'))
            else:
                click.echo(click.style(update_result.message, fg='red'))
        else:
            click.echo(click.style('failed to store model'.format(result), fg='red'))

    except ApiException as e:
        click.echo(click.style(json.loads(e.body)['detail'], fg='red'))


@cli.command()
@click.option('--user', multiple=True, help="list of users (testuser, root), [multiple]: --user a --user b")
@click.option('--metric', multiple=True, help="list of metrics (prec, recall, f1, accuracy),[multiple]: --metric f1 "
                                              "--metric acc")
@click.option('--sort', help="specify one metric to sort the results", default=None)
@click.option('--event_type', default='test', help="specify one metric to sort the results")
@click.option('--n', help='number of experiments', type=int)
@click.option('--output', help='output file (csv)')
@click.option('--output_fields', multiple=True, help="which field(s) you want to see in output",
              default=['username', 'eid'])
@click.argument('task')
@click.argument('label')
def searchlabel(task, label, user, metric, sort, event_type, n, output, output_fields):
    """
    Shows the results for all experiments for a particular label. Optionally filter out by user(s), metric(s),
    or sort by one metric. Shows the results on the test data by default, provide event_type (train/valid/test)
    to see for other events.
    """
    event_type = EVENT_TYPES[event_type]
    ServerManager.get()
    try:
        result = ServerManager.api.list_experiments_by_prop(task, label=label, user=user, metric=metric,
                                                            sort=sort, event_type=event_type)
        prop_name_loc = {k: i for i, k in enumerate(output_fields)}
        result_df = experiment_list_to_df(exps=result, prop_name_loc=prop_name_loc, event_type=event_type)
        if n != -1:
            result_df = result_df.head(n)
        if output is None:
            click.echo(result_df)
        else:
            result_df.to_csv(output)
    except ApiException as e:
        click.echo(click.style(json.loads(e.body)['detail'], fg='red'))


if __name__ == "__main__":
    cli()



# coding: utf-8

"""
    xpctl

    This is a sample xpctl  server.  You can find out more about xpctl at [baseline](https://github.com/dpressel/baseline/blob/master/docs/xpctl.md).  # noqa: E501

    OpenAPI spec version: 1.0.0
    Contact: apiteam@swagger.io
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class Result(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'metric': 'str',
        'value': 'float',
        'tick_type': 'str',
        'tick': 'int',
        'phase': 'str'
    }

    attribute_map = {
        'metric': 'metric',
        'value': 'value',
        'tick_type': 'tick_type',
        'tick': 'tick',
        'phase': 'phase'
    }

    def __init__(self, metric=None, value=None, tick_type=None, tick=None, phase=None):  # noqa: E501
        """Result - a model defined in Swagger"""  # noqa: E501

        self._metric = None
        self._value = None
        self._tick_type = None
        self._tick = None
        self._phase = None
        self.discriminator = None

        self.metric = metric
        self.value = value
        self.tick_type = tick_type
        self.tick = tick
        self.phase = phase

    @property
    def metric(self):
        """Gets the metric of this Result.  # noqa: E501


        :return: The metric of this Result.  # noqa: E501
        :rtype: str
        """
        return self._metric

    @metric.setter
    def metric(self, metric):
        """Sets the metric of this Result.


        :param metric: The metric of this Result.  # noqa: E501
        :type: str
        """
        if metric is None:
            raise ValueError("Invalid value for `metric`, must not be `None`")  # noqa: E501

        self._metric = metric

    @property
    def value(self):
        """Gets the value of this Result.  # noqa: E501


        :return: The value of this Result.  # noqa: E501
        :rtype: float
        """
        return self._value

    @value.setter
    def value(self, value):
        """Sets the value of this Result.


        :param value: The value of this Result.  # noqa: E501
        :type: float
        """
        if value is None:
            raise ValueError("Invalid value for `value`, must not be `None`")  # noqa: E501

        self._value = value

    @property
    def tick_type(self):
        """Gets the tick_type of this Result.  # noqa: E501


        :return: The tick_type of this Result.  # noqa: E501
        :rtype: str
        """
        return self._tick_type

    @tick_type.setter
    def tick_type(self, tick_type):
        """Sets the tick_type of this Result.


        :param tick_type: The tick_type of this Result.  # noqa: E501
        :type: str
        """
        if tick_type is None:
            raise ValueError("Invalid value for `tick_type`, must not be `None`")  # noqa: E501

        self._tick_type = tick_type

    @property
    def tick(self):
        """Gets the tick of this Result.  # noqa: E501


        :return: The tick of this Result.  # noqa: E501
        :rtype: int
        """
        return self._tick

    @tick.setter
    def tick(self, tick):
        """Sets the tick of this Result.


        :param tick: The tick of this Result.  # noqa: E501
        :type: int
        """
        if tick is None:
            raise ValueError("Invalid value for `tick`, must not be `None`")  # noqa: E501

        self._tick = tick

    @property
    def phase(self):
        """Gets the phase of this Result.  # noqa: E501


        :return: The phase of this Result.  # noqa: E501
        :rtype: str
        """
        return self._phase

    @phase.setter
    def phase(self, phase):
        """Sets the phase of this Result.


        :param phase: The phase of this Result.  # noqa: E501
        :type: str
        """
        if phase is None:
            raise ValueError("Invalid value for `phase`, must not be `None`")  # noqa: E501

        self._phase = phase

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Result, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Result):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from xpctl.xpserver.models.base_model_ import Model
from xpctl.xpserver import util


class Response(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, code=None, message=None, response_type=None):  # noqa: E501
        """Response - a model defined in Swagger

        :param code: The code of this Response.  # noqa: E501
        :type code: int
        :param message: The message of this Response.  # noqa: E501
        :type message: str
        :param response_type: The response_type of this Response.  # noqa: E501
        :type response_type: str
        """
        self.swagger_types = {
            'code': int,
            'message': str,
            'response_type': str
        }

        self.attribute_map = {
            'code': 'code',
            'message': 'message',
            'response_type': 'response_type'
        }

        self._code = code
        self._message = message
        self._response_type = response_type

    @classmethod
    def from_dict(cls, dikt):
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The Response of this Response.  # noqa: E501
        :rtype: Response
        """
        return util.deserialize_model(dikt, cls)

    @property
    def code(self):
        """Gets the code of this Response.


        :return: The code of this Response.
        :rtype: int
        """
        return self._code

    @code.setter
    def code(self, code):
        """Sets the code of this Response.


        :param code: The code of this Response.
        :type code: int
        """
        if code is None:
            raise ValueError("Invalid value for `code`, must not be `None`")  # noqa: E501

        self._code = code

    @property
    def message(self):
        """Gets the message of this Response.


        :return: The message of this Response.
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """Sets the message of this Response.


        :param message: The message of this Response.
        :type message: str
        """
        if message is None:
            raise ValueError("Invalid value for `message`, must not be `None`")  # noqa: E501

        self._message = message

    @property
    def response_type(self):
        """Gets the response_type of this Response.


        :return: The response_type of this Response.
        :rtype: str
        """
        return self._response_type

    @response_type.setter
    def response_type(self, response_type):
        """Sets the response_type of this Response.


        :param response_type: The response_type of this Response.
        :type response_type: str
        """
        allowed_values = ["success", "error"]  # noqa: E501
        if response_type not in allowed_values:
            raise ValueError(
                "Invalid value for `response_type` ({0}), must be one of {1}"
                .format(response_type, allowed_values)
            )

        self._response_type = response_type



# coding: utf-8

from __future__ import absolute_import

from flask import json
from six import BytesIO

from xpctl.xpserver.models.experiment import Experiment  # noqa: E501
from xpctl.xpserver.models.experiment_aggregate import ExperimentAggregate  # noqa: E501
from xpctl.xpserver.models.response import Response  # noqa: E501
from xpctl.xpserver.models.task_summary import TaskSummary  # noqa: E501
from xpctl.xpserver.test import BaseTestCase


class TestXpctlController(BaseTestCase):
    """XpctlController integration test stubs"""

    def test_config2json(self):
        """Test case for config2json

        get config for sha1
        """
        response = self.client.open(
            '/v2/config2json/{task}/{sha1}'.format(task='task_example', sha1='sha1_example'),
            method='GET')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_experiment_details(self):
        """Test case for experiment_details

        Find experiment by id
        """
        query_string = [('event_type', 'event_type_example'),
                        ('metric', 'metric_example')]
        response = self.client.open(
            '/v2/{task}/{eid}'.format(task='task_example', eid='eid_example'),
            method='GET',
            query_string=query_string)
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_get_model_location(self):
        """Test case for get_model_location

        get model loc for experiment
        """
        response = self.client.open(
            '/v2/getmodelloc/{task}/{eid}'.format(task='task_example', eid='eid_example'),
            method='GET')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_get_results_by_prop(self):
        """Test case for get_results_by_prop

        Find results by property and value
        """
        query_string = [('eid', 'eid_example'),
                        ('sha1', 'sha1_example'),
                        ('dataset', 'dataset_example'),
                        ('label', 'label_example'),
                        ('reduction_dim', 'reduction_dim_example'),
                        ('metric', 'metric_example'),
                        ('sort', 'sort_example'),
                        ('numexp_reduction_dim', 56),
                        ('event_type', 'event_type_example')]
        response = self.client.open(
            '/v2/results/{task}'.format(task='task_example'),
            method='GET',
            query_string=query_string)
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_list_experiments_by_prop(self):
        """Test case for list_experiments_by_prop

        list all experiments for this property and value
        """
        query_string = [('eid', 'eid_example'),
                        ('sha1', 'sha1_example'),
                        ('dataset', 'dataset_example'),
                        ('label', 'label_example'),
                        ('user', 'user_example'),
                        ('metric', 'metric_example'),
                        ('sort', 'sort_example'),
                        ('event_type', 'event_type_example')]
        response = self.client.open(
            '/v2/find/{task}'.format(task='task_example'),
            method='GET',
            query_string=query_string)
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_put_result(self):
        """Test case for put_result

        Add a new experiment in database
        """
        experiment = Experiment()
        query_string = [('user', 'user_example'),
                        ('label', 'label_example')]
        response = self.client.open(
            '/v2/put/{task}'.format(task='task_example'),
            method='POST',
            data=json.dumps(experiment),
            content_type='application/json',
            query_string=query_string)
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_remove_experiment(self):
        """Test case for remove_experiment

        delete an experiment from the database
        """
        response = self.client.open(
            '/v2/delete/{task}/{eid}'.format(task='task_example', eid='eid_example'),
            method='GET',
            content_type='application/json')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_summary(self):
        """Test case for summary

        get summary for task
        """
        response = self.client.open(
            '/v2/summary/',
            method='GET')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_task_summary(self):
        """Test case for task_summary

        get summary for task
        """
        response = self.client.open(
            '/v2/summary/{task}/'.format(task='task_example'),
            method='GET')
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))

    def test_update_property(self):
        """Test case for update_property

        update property for an experiment
        """
        query_string = [('prop', 'prop_example'),
                        ('value', 'value_example')]
        response = self.client.open(
            '/v2/update/{task}/{eid}/'.format(task='task_example', eid='eid_example'),
            method='GET',
            content_type='application/json',
            query_string=query_string)
        self.assert200(response,
                       'Response body is : ' + response.data.decode('utf-8'))


if __name__ == '__main__':
    import unittest
    unittest.main()



# coding: utf-8

"""
    xpctl

    This is a sample xpctl  server.  You can find out more about xpctl at [baseline](https://github.com/dpressel/baseline/blob/master/docs/xpctl.md).  # noqa: E501

    OpenAPI spec version: 1.0.0
    Contact: apiteam@swagger.io
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from xpctl.xpclient.api_client import ApiClient


class XpctlApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def config2json(self, task, sha1, **kwargs):  # noqa: E501
        """get config for sha1  # noqa: E501

        config for sha1  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.config2json(task, sha1, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str task: task (required)
        :param str sha1: sha1 (required)
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.config2json_with_http_info(task, sha1, **kwargs)  # noqa: E501
        else:
            (data) = self.config2json_with_http_info(task, sha1, **kwargs)  # noqa: E501
            return data

    def config2json_with_http_info(self, task, sha1, **kwargs):  # noqa: E501
        """get config for sha1  # noqa: E501

        config for sha1  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.config2json_with_http_info(task, sha1, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str task: task (required)
        :param str sha1: sha1 (required)
        :return: object
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['task', 'sha1']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method config2json" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'task' is set
        if ('task' not in params or
                params['task'] is None):
            raise ValueError("Missing the required parameter `task` when calling `config2json`")  # noqa: E501
        # verify the required parameter 'sha1' is set
        if ('sha1' not in params or
                params['sha1'] is None):
            raise ValueError("Missing the required parameter `sha1` when calling `config2json`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'task' in params:
            path_params['task'] = params['task']  # noqa: E501
        if 'sha1' in params:
            path_params['sha1'] = params['sha1']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/config2json/{task}/{sha1}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='object',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def experiment_details(self, task, eid, **kwargs):  # noqa: E501
        """Find experiment by id  # noqa: E501

        Returns a single experiment  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.experiment_details(task, eid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str task: task name (required)
        :param str eid: ID of experiment to return (required)
        :param str event_type: 
        :param list[str] metric: 
        :return: Experiment
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.experiment_details_with_http_info(task, eid, **kwargs)  # noqa: E501
        else:
            (data) = self.experiment_details_with_http_info(task, eid, **kwargs)  # noqa: E501
            return data

    def experiment_details_with_http_info(self, task, eid, **kwargs):  # noqa: E501
        """Find experiment by id  # noqa: E501

        Returns a single experiment  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.experiment_details_with_http_info(task, eid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str task: task name (required)
        :param str eid: ID of experiment to return (required)
        :param str event_type: 
        :param list[str] metric: 
        :return: Experiment
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['task', 'eid', 'event_type', 'metric']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method experiment_details" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'task' is set
        if ('task' not in params or
                params['task'] is None):
            raise ValueError("Missing the required parameter `task` when calling `experiment_details`")  # noqa: E501
        # verify the required parameter 'eid' is set
        if ('eid' not in params or
                params['eid'] is None):
            raise ValueError("Missing the required parameter `eid` when calling `experiment_details`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'task' in params:
            path_params['task'] = params['task']  # noqa: E501
        if 'eid' in params:
            path_params['eid'] = params['eid']  # noqa: E501

        query_params = []
        if 'event_type' in params:
            query_params.append(('event_type', params['event_type']))  # noqa: E501
        if 'metric' in params:
            query_params.append(('metric', params['metric']))  # noqa: E501
            collection_formats['metric'] = 'csv'  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/{task}/{eid}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Experiment',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_model_location(self, task, eid, **kwargs):  # noqa: E501
        """get model loc for experiment  # noqa: E501

        get model loc for experiment  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_model_location(task, eid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str task: task (required)
        :param str eid: experiment id (required)
        :return: Response
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_model_location_with_http_info(task, eid, **kwargs)  # noqa: E501
        else:
            (data) = self.get_model_location_with_http_info(task, eid, **kwargs)  # noqa: E501
            return data

    def get_model_location_with_http_info(self, task, eid, **kwargs):  # noqa: E501
        """get model loc for experiment  # noqa: E501

        get model loc for experiment  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_model_location_with_http_info(task, eid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str task: task (required)
        :param str eid: experiment id (required)
        :return: Response
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['task', 'eid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_model_location" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'task' is set
        if ('task' not in params or
                params['task'] is None):
            raise ValueError("Missing the required parameter `task` when calling `get_model_location`")  # noqa: E501
        # verify the required parameter 'eid' is set
        if ('eid' not in params or
                params['eid'] is None):
            raise ValueError("Missing the required parameter `eid` when calling `get_model_location`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'task' in params:
            path_params['task'] = params['task']  # noqa: E501
        if 'eid' in params:
            path_params['eid'] = params['eid']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/getmodelloc/{task}/{eid}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Response',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_results_by_prop(self, task, **kwargs):  # noqa: E501
        """Find results by property and value  # noqa: E501

        Find results by property and value  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_results_by_prop(task, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str task: task name (required)
        :param str eid: eid
        :param str sha1: sha1
        :param str dataset: dataset, e.g. SST2
        :param str label: label
        :param str reduction_dim: which dimension to reduce on, default=sha1
        :param list[str] metric: metric
        :param str sort: metric to sort results on
        :param int numexp_reduction_dim: max number of experiments in an aggregate group
        :param str event_type: train/dev/test
        :return: list[ExperimentAggregate]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_results_by_prop_with_http_info(task, **kwargs)  # noqa: E501
        else:
            (data) = self.get_results_by_prop_with_http_info(task, **kwargs)  # noqa: E501
            return data

    def get_results_by_prop_with_http_info(self, task, **kwargs):  # noqa: E501
        """Find results by property and value  # noqa: E501

        Find results by property and value  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_results_by_prop_with_http_info(task, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str task: task name (required)
        :param str eid: eid
        :param str sha1: sha1
        :param str dataset: dataset, e.g. SST2
        :param str label: label
        :param str reduction_dim: which dimension to reduce on, default=sha1
        :param list[str] metric: metric
        :param str sort: metric to sort results on
        :param int numexp_reduction_dim: max number of experiments in an aggregate group
        :param str event_type: train/dev/test
        :return: list[ExperimentAggregate]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['task', 'eid', 'sha1', 'dataset', 'label', 'reduction_dim', 'metric', 'sort', 'numexp_reduction_dim', 'event_type']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_results_by_prop" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'task' is set
        if ('task' not in params or
                params['task'] is None):
            raise ValueError("Missing the required parameter `task` when calling `get_results_by_prop`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'task' in params:
            path_params['task'] = params['task']  # noqa: E501

        query_params = []
        if 'eid' in params:
            query_params.append(('eid', params['eid']))  # noqa: E501
        if 'sha1' in params:
            query_params.append(('sha1', params['sha1']))  # noqa: E501
        if 'dataset' in params:
            query_params.append(('dataset', params['dataset']))  # noqa: E501
        if 'label' in params:
            query_params.append(('label', params['label']))  # noqa: E501
        if 'reduction_dim' in params:
            query_params.append(('reduction_dim', params['reduction_dim']))  # noqa: E501
        if 'metric' in params:
            query_params.append(('metric', params['metric']))  # noqa: E501
            collection_formats['metric'] = 'csv'  # noqa: E501
        if 'sort' in params:
            query_params.append(('sort', params['sort']))  # noqa: E501
        if 'numexp_reduction_dim' in params:
            query_params.append(('numexp_reduction_dim', params['numexp_reduction_dim']))  # noqa: E501
        if 'event_type' in params:
            query_params.append(('event_type', params['event_type']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/results/{task}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[ExperimentAggregate]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def list_experiments_by_prop(self, task, **kwargs):  # noqa: E501
        """list all experiments for this property and value  # noqa: E501

        list all experiments for this property (sha1/ username) and value (1cd21df91770b4dbed64a683558b062e3dee61f0/ dpressel)  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.list_experiments_by_prop(task, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str task: task name (required)
        :param str eid: eid
        :param str sha1: sha1
        :param str dataset: dataset, e.g. SST2
        :param str label: label
        :param list[str] user:
        :param list[str] metric:
        :param str sort:
        :param str event_type:
        :return: list[Experiment]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.list_experiments_by_prop_with_http_info(task, **kwargs)  # noqa: E501
        else:
            (data) = self.list_experiments_by_prop_with_http_info(task, **kwargs)  # noqa: E501
            return data

    def list_experiments_by_prop_with_http_info(self, task, **kwargs):  # noqa: E501
        """list all experiments for this property and value  # noqa: E501

        list all experiments for this property (sha1/ username) and value (1cd21df91770b4dbed64a683558b062e3dee61f0/ dpressel)  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.list_experiments_by_prop_with_http_info(task, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str task: task name (required)
        :param str eid: eid
        :param str sha1: sha1
        :param str dataset: dataset, e.g. SST2
        :param str label: label
        :param list[str] user:
        :param list[str] metric:
        :param str sort:
        :param str event_type:
        :return: list[Experiment]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['task', 'eid', 'sha1', 'dataset', 'label', 'user', 'metric', 'sort', 'event_type']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_experiments_by_prop" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'task' is set
        if ('task' not in params or
                params['task'] is None):
            raise ValueError("Missing the required parameter `task` when calling `list_experiments_by_prop`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'task' in params:
            path_params['task'] = params['task']  # noqa: E501

        query_params = []
        if 'eid' in params:
            query_params.append(('eid', params['eid']))  # noqa: E501
        if 'sha1' in params:
            query_params.append(('sha1', params['sha1']))  # noqa: E501
        if 'dataset' in params:
            query_params.append(('dataset', params['dataset']))  # noqa: E501
        if 'label' in params:
            query_params.append(('label', params['label']))  # noqa: E501
        if 'user' in params:
            query_params.append(('user', params['user']))  # noqa: E501
            collection_formats['user'] = 'csv'  # noqa: E501
        if 'metric' in params:
            query_params.append(('metric', params['metric']))  # noqa: E501
            collection_formats['metric'] = 'csv'  # noqa: E501
        if 'sort' in params:
            query_params.append(('sort', params['sort']))  # noqa: E501
        if 'event_type' in params:
            query_params.append(('event_type', params['event_type']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/find/{task}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[Experiment]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def put_result(self, task, experiment, **kwargs):  # noqa: E501
        """Add a new experiment in database  # noqa: E501

          # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.put_result(task, experiment, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str task:  (required)
        :param Experiment experiment: (required)
        :param str user: 
        :param str label: 
        :return: Response
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.put_result_with_http_info(task, experiment, **kwargs)  # noqa: E501
        else:
            (data) = self.put_result_with_http_info(task, experiment, **kwargs)  # noqa: E501
            return data

    def put_result_with_http_info(self, task, experiment, **kwargs):  # noqa: E501
        """Add a new experiment in database  # noqa: E501

          # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.put_result_with_http_info(task, experiment, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str task:  (required)
        :param Experiment experiment: (required)
        :param str user: 
        :param str label: 
        :return: Response
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['task', 'experiment', 'user', 'label']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method put_result" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'task' is set
        if ('task' not in params or
                params['task'] is None):
            raise ValueError("Missing the required parameter `task` when calling `put_result`")  # noqa: E501
        # verify the required parameter 'experiment' is set
        if ('experiment' not in params or
                params['experiment'] is None):
            raise ValueError("Missing the required parameter `experiment` when calling `put_result`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'task' in params:
            path_params['task'] = params['task']  # noqa: E501

        query_params = []
        if 'user' in params:
            query_params.append(('user', params['user']))  # noqa: E501
        if 'label' in params:
            query_params.append(('label', params['label']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'experiment' in params:
            body_params = params['experiment']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/put/{task}', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Response',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def remove_experiment(self, task, eid, **kwargs):  # noqa: E501
        """delete an experiment from the database  # noqa: E501

          # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.remove_experiment(task, eid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str task:  (required)
        :param str eid:  (required)
        :return: Response
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.remove_experiment_with_http_info(task, eid, **kwargs)  # noqa: E501
        else:
            (data) = self.remove_experiment_with_http_info(task, eid, **kwargs)  # noqa: E501
            return data

    def remove_experiment_with_http_info(self, task, eid, **kwargs):  # noqa: E501
        """delete an experiment from the database  # noqa: E501

          # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.remove_experiment_with_http_info(task, eid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str task:  (required)
        :param str eid:  (required)
        :return: Response
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['task', 'eid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method remove_experiment" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'task' is set
        if ('task' not in params or
                params['task'] is None):
            raise ValueError("Missing the required parameter `task` when calling `remove_experiment`")  # noqa: E501
        # verify the required parameter 'eid' is set
        if ('eid' not in params or
                params['eid'] is None):
            raise ValueError("Missing the required parameter `eid` when calling `remove_experiment`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'task' in params:
            path_params['task'] = params['task']  # noqa: E501
        if 'eid' in params:
            path_params['eid'] = params['eid']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/delete/{task}/{eid}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Response',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def summary(self, **kwargs):  # noqa: E501
        """get summary for task  # noqa: E501

        summary for task  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.summary(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: list[TaskSummary]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.summary_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.summary_with_http_info(**kwargs)  # noqa: E501
            return data

    def summary_with_http_info(self, **kwargs):  # noqa: E501
        """get summary for task  # noqa: E501

        summary for task  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.summary_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: list[TaskSummary]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = []  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method summary" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/summary/', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[TaskSummary]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def task_summary(self, task, **kwargs):  # noqa: E501
        """get summary for task  # noqa: E501

        summary for task  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.task_summary(task, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str task: task (required)
        :return: TaskSummary
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.task_summary_with_http_info(task, **kwargs)  # noqa: E501
        else:
            (data) = self.task_summary_with_http_info(task, **kwargs)  # noqa: E501
            return data

    def task_summary_with_http_info(self, task, **kwargs):  # noqa: E501
        """get summary for task  # noqa: E501

        summary for task  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.task_summary_with_http_info(task, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str task: task (required)
        :return: TaskSummary
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['task']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method task_summary" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'task' is set
        if ('task' not in params or
                params['task'] is None):
            raise ValueError("Missing the required parameter `task` when calling `task_summary`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'task' in params:
            path_params['task'] = params['task']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/summary/{task}/', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='TaskSummary',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def update_property(self, task, eid, prop, value, **kwargs):  # noqa: E501
        """update property for an experiment  # noqa: E501

          # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.update_property(task, eid, prop, value, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str task:  (required)
        :param str eid:  (required)
        :param str prop:  (required)
        :param str value:  (required)
        :return: Response
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.update_property_with_http_info(task, eid, prop, value, **kwargs)  # noqa: E501
        else:
            (data) = self.update_property_with_http_info(task, eid, prop, value, **kwargs)  # noqa: E501
            return data

    def update_property_with_http_info(self, task, eid, prop, value, **kwargs):  # noqa: E501
        """update property for an experiment  # noqa: E501

          # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.update_property_with_http_info(task, eid, prop, value, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str task:  (required)
        :param str eid:  (required)
        :param str prop:  (required)
        :param str value:  (required)
        :return: Response
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['task', 'eid', 'prop', 'value']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method update_property" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'task' is set
        if ('task' not in params or
                params['task'] is None):
            raise ValueError("Missing the required parameter `task` when calling `update_property`")  # noqa: E501
        # verify the required parameter 'eid' is set
        if ('eid' not in params or
                params['eid'] is None):
            raise ValueError("Missing the required parameter `eid` when calling `update_property`")  # noqa: E501
        # verify the required parameter 'prop' is set
        if ('prop' not in params or
                params['prop'] is None):
            raise ValueError("Missing the required parameter `prop` when calling `update_property`")  # noqa: E501
        # verify the required parameter 'value' is set
        if ('value' not in params or
                params['value'] is None):
            raise ValueError("Missing the required parameter `value` when calling `update_property`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'task' in params:
            path_params['task'] = params['task']  # noqa: E501
        if 'eid' in params:
            path_params['eid'] = params['eid']  # noqa: E501

        query_params = []
        if 'prop' in params:
            query_params.append(('prop', params['prop']))  # noqa: E501
        if 'value' in params:
            query_params.append(('value', params['value']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = []  # noqa: E501

        return self.api_client.call_api(
            '/update/{task}/{eid}/', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Response',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



from copy import deepcopy
from collections import namedtuple
from itertools import groupby
import json
import os
import numpy as np
from flask import abort

from xpctl.utils import write_config_file

from xpctl.xpserver.models import Experiment as ServerExperiment
from xpctl.xpserver.models import Result as ServerResult
from xpctl.xpserver.models import ExperimentAggregate as ServerExperimentAggregate
from xpctl.xpserver.models import Response as ServerResponse
from xpctl.xpserver.models import TaskSummary as ServerTaskSummary
from xpctl.xpserver.models import AggregateResult as ServerAggregateResult
from xpctl.xpserver.models import AggregateResultValues


TRAIN_EVENT = 'train_events'
VALID_EVENT = 'valid_events'
TEST_EVENT = 'test_events'
EVENT_TYPES = [TRAIN_EVENT, VALID_EVENT, TEST_EVENT]


METRICS_SORT_ASCENDING = ['avg_loss', 'perplexity']


class Result(object):
    def __init__(self, metric, value, tick_type, tick, phase):
        super(Result, self).__init__()
        self.metric = metric
        self.value = value
        self.tick_type = tick_type
        self.tick = tick
        self.phase = phase

    def get_prop(self, field):
        if field not in self.__dict__:
            raise ValueError('{} does not have a property {}'.format(self.__class__, field))
        return self.__dict__[field]


class AggregateResult(object):
    def __init__(self, metric, values):
        super(AggregateResult, self).__init__()
        self.metric = metric
        self.values = values

    def get_prop(self, field):
        if field not in self.__dict__:
            raise ValueError('{} does not have a property {}'.format(self.__class__, field))
        return self.__dict__[field]


class Experiment(object):
    """ an experiment"""
    def __init__(self, train_events, valid_events, test_events, task, eid, username, hostname, config, exp_date, label,
                 dataset, sha1, version):
        super(Experiment, self).__init__()
        self.task = task
        self.train_events = train_events if train_events is not None else []
        self.valid_events = valid_events if valid_events is not None else []
        self.test_events = test_events if test_events is not None else []
        self.eid = eid
        self.username = username
        self.hostname = hostname
        self.config = config
        self.exp_date = exp_date
        self.label = label
        self.dataset = dataset
        self.sha1 = sha1
        self.version = version

    def get_prop(self, field):
        if field not in self.__dict__:
            raise ValueError('{} does not have a property {}'.format(self.__class__, field))
        return self.__dict__[field]

    def add_result(self, result, event_type):
        if event_type == TRAIN_EVENT:
            self.train_events.append(result)
        elif event_type == VALID_EVENT:
            self.valid_events.append(result)
        elif event_type == TEST_EVENT:
            self.test_events.append(result)
        else:
            raise NotImplementedError('no handler for event type: [{}]'.format(event_type))


class ExperimentSet(object):
    """ a list of experiment objects"""
    def __init__(self, data):
        super(ExperimentSet, self).__init__()
        self.data = data if data else []  # this should ideally be a set but the items are not hashable
        self.length = len(self.data)
    
    def __getitem__(self, i):
        return self.data[i]
    
    def __iter__(self):
        for i in range(self.length):
            yield self.data[i]
    
    def __len__(self):
        return self.length

    def add_data(self, datum):
        """
        add a experiment data point
        :param datum:
        :return:
        """
        self.data.append(datum)
        self.length += 1
        
    def groupby(self, key):
        """ group the data points by key"""
        data_groups = {}
        if len(self.data) == 0:
            raise RuntimeError('Trying to group empty experiment set')
        task = self.data[0].get_prop('task')
        for datum in self.data:
            if datum.get_prop('task') != task:
                raise RuntimeError('Should not be grouping two experiments from different tasks')
            field = datum.get_prop(key)
            if field not in data_groups:
                data_groups[field] = ExperimentSet([datum])
            else:
                data_groups[field].add_data(datum)
        return ExperimentGroup(data_groups, key, task)

    def sort(self, key, reverse=True):
        """
        you can only sort when event_type is test, because there is only one data point
        :param key: metric to sort on
        :param reverse: reverse=True always except when key is avg_loss
        :return:
        """
        if key is None or key == 'None':
            return self
        test_results = [(index, [y for y in x.get_prop(TEST_EVENT) if y.metric == key][0]) for index, x in
                        enumerate(self.data)]
        test_results.sort(key=lambda x: x[1].value, reverse=reverse)
        final_results = []
        for index, _ in test_results:
            final_results.append(self.data[index])
        return ExperimentSet(data=final_results)


class ExperimentGroup(object):
    """ a group of resultset objects"""
    def __init__(self, grouped_experiments, reduction_dim, task):
        super(ExperimentGroup, self).__init__()
        self.grouped_experiments = grouped_experiments
        self.reduction_dim = reduction_dim
        self.task = task

    def items(self):
        return self.grouped_experiments.items()

    def keys(self):
        return self.grouped_experiments.keys()

    def values(self):
        return self.grouped_experiments.values()

    def __iter__(self):
        for k, v in self.grouped_experiments.items():
            yield (k, v)

    def get(self, key):
        return self.grouped_experiments.get(key)

    def __len__(self):
        return len(self.grouped_experiments.keys())

    def reduce(self, aggregate_fns, event_type=TEST_EVENT, prop_dict={}):
        """ aggregate results across a result group
        :param aggregate_fns: aggregating functions
        :param event_type: train/valid/test event
        :param prop_dict: Experiments in this group are expected to share some properties (since they are typically a
        query results from the database. This dict stores the shared properties and their values. The
        ExperimentAggregate created by the this reduce function will be labeld by these properties and values.
        """
        data = {}
        num_experiments = {}
        for reduction_dim_value, experiments in self.grouped_experiments.items():
            for dataset, _experiments in groupby(experiments, key=lambda exp: exp.dataset):
                _experiments = list(_experiments)
                num_experiments[reduction_dim_value] = {dataset: len(_experiments)}
                data[reduction_dim_value] = {dataset: {} for dataset in prop_dict['dataset']}
                for experiment in _experiments:
                    results = experiment.get_prop(event_type)
                    for result in results:
                        if result.metric not in data[reduction_dim_value][dataset]:
                            data[reduction_dim_value][dataset][result.metric] = [result.value]
                        else:
                            data[reduction_dim_value][dataset][result.metric].append(result.value)
        # for each reduction_dim_value, only one dataset can have metrics, the others are empty
        _data = {reduction_dim_value: {} for reduction_dim_value in data}
        for reduction_dim_value in data:
            for dataset in data[reduction_dim_value]:
                if data[reduction_dim_value][dataset]:
                    _data[reduction_dim_value][dataset] = data[reduction_dim_value][dataset]
        data = _data
        # for each reduction dim value, (say when sha1 = x), all data[x][metric] lists should have the same length.
        for reduction_dim_value in data:
            for dataset in data[reduction_dim_value]:
                lengths = []
                for metric in data[reduction_dim_value][dataset]:
                    lengths.append(len(data[reduction_dim_value][dataset][metric]))
                try:
                    assert len(set(lengths)) == 1
                except AssertionError:
                    raise AssertionError('when reducing experiments over {}, for {}={}, the number of results are '
                                         'not the same over all metrics'.format(self.reduction_dim,
                                                                                self.reduction_dim,
                                                                                reduction_dim_value))

        aggregate_resultset = ExperimentAggregateSet(data=[])
        del prop_dict['dataset']  # prop_dict must have dataset
        for reduction_dim_value in data:
            for dataset in data[reduction_dim_value]:
                    prop_dict.update({'dataset': dataset})
                    values = {}
                    d = {
                        self.reduction_dim: reduction_dim_value,
                        'num_exps': num_experiments[reduction_dim_value][dataset],
                        **prop_dict
                    }
                    agr = deepcopy(ExperimentAggregate(task=self.task, **d))
                    for metric in data[reduction_dim_value][dataset]:
                        for fn_name, fn in aggregate_fns.items():
                            agg_value = fn(data[reduction_dim_value][dataset][metric])
                            print(metric, dataset, fn_name, data[reduction_dim_value][dataset][metric], agg_value)
                            values[fn_name] = agg_value
                        agr.add_result(deepcopy(AggregateResult(metric=metric, values=values)), event_type=event_type)
                    aggregate_resultset.add_data(agr)
        return aggregate_resultset


class ExperimentAggregate(object):
    """ a result data point"""
    def __init__(self, task, train_events=[], valid_events=[], test_events=[], **kwargs):
        super(ExperimentAggregate, self).__init__()
        self.train_events = train_events if train_events is not None else []
        self.valid_events = valid_events if valid_events is not None else []
        self.test_events = test_events if test_events is not None else []
        self.task = task
        self.num_exps = kwargs.get('num_exps')
        self.eid = kwargs.get('eid')
        self.username = kwargs.get('username')
        self.label = kwargs.get('label')
        self.dataset = kwargs.get('dataset')
        self.exp_date = kwargs.get('exp_date')
        self.sha1 = kwargs.get('sha1')

    def get_prop(self, field):
        return self.__dict__[field]

    def add_result(self, aggregate_result, event_type):
        if event_type == TRAIN_EVENT:
            self.train_events.append(aggregate_result)
        elif event_type == VALID_EVENT:
            self.valid_events.append(aggregate_result)
        elif event_type == TEST_EVENT:
            self.test_events.append(aggregate_result)
        else:
            raise NotImplementedError('no handler for event type: [{}]'.format(event_type))


class ExperimentAggregateSet(object):
    """ a list of aggregate result objects"""
    def __init__(self, data):
        super(ExperimentAggregateSet, self).__init__()
        self.data = data if data else []
        self.length = len(self.data)

    def add_data(self, data_point):
        """
        add a aggregateresult data point
        :param data_point:
        :return:
        """
        self.data.append(data_point)
        self.length += 1

    # TODO: add property annotations
    def __getitem__(self, i):
        return self.data[i]

    def __iter__(self):
        for i in range(self.length):
            yield self.data[i]

    def __len__(self):
        return self.length

    def sort(self, metric, aggregate_fn='avg', reverse=True):
        """
        you can only sort when event_type is test, because there is only one data point
        :param metric: metric to sort on
        :param aggregate_fn: this is an aggregate result, you have values for for different aggregate_fns. choose one
        :param reverse: reverse=True always except when key is avg_loss, perplexity
        :return:
        """
        if metric is None:
            return self
        test_results = [(index, [y for y in x.get_prop(TEST_EVENT) if y.metric == metric][0]) for index, x in
                        enumerate(self.data)]
        test_results.sort(key=lambda x: x[1].values[aggregate_fn], reverse=reverse)
        final_results = []
        for index, _ in test_results:
            final_results.append(self.data[index])
        return ExperimentSet(data=final_results)


class TaskDatasetSummary(object):
    """ How many users experimented with this dataset in the given task?"""
    def __init__(self, task, dataset, experiment_set, user_num_exps=None):
        super(TaskDatasetSummary, self).__init__()
        self.task = task
        self.dataset = dataset
        if user_num_exps is not None:
            self.user_num_exps = user_num_exps
        else:
            exp_groups = experiment_set.groupby('username')
            self.user_num_exps = {username: len(exp_group)for username, exp_group in exp_groups}


class TaskDatasetSummarySet(object):
    """ a list of TaskDatasetSummary objects."""
    def __init__(self, task, data):
        self.task = task
        self.data = data

    def groupby(self):
        """ group the TaskDatasetSummary objects. """
        d = {}
        for tdsummary in self.data:
            dataset = tdsummary.dataset
            d[dataset] = []
            for username in tdsummary.user_num_exps:
                d[dataset].append((username, tdsummary.user_num_exps[username]))

        return TaskSummary(self.task, d)


class TaskSummary(object):
    def __init__(self, task, summary):
        self.task = task
        self.summary = summary


class BackendResponse(object):
    def __init__(self, message, response_type, code=550):
        super(BackendResponse, self).__init__()
        self.message = message
        self.response_type = response_type
        self.code = code


class BackendError(BackendResponse):
    def __init__(self, message, response_type="error", code=550):
        super(BackendError, self).__init__(message, response_type, code)
        self.message = message
        self.response_type = response_type
        self.code = code


class BackendSuccess(BackendResponse):
    def __init__(self, message, response_type="success", code=250):
        super(BackendSuccess, self).__init__(message, response_type, code)
        self.message = message
        self.response_type = response_type
        self.code = code


def log2json(log_file):
    s = []
    with open(log_file) as f:
        for line in f:
            x = line.replace("'", '"')
            s.append(json.loads(x))
    return s


def json2log(events, log_file):
    with open(log_file, 'w') as wf:
        for event in events:
            wf.write(json.dumps(event)+'\n')


def get_experiment_label(config_obj, task, **kwargs):
    if kwargs.get('label', None) is not None:
        return kwargs['label']
    if 'description' in config_obj:
        return config_obj['description']
    else:
        model_type = config_obj.get('model_type', 'default')
        backend = config_obj.get('backend', 'tensorflow')
        return "{}-{}-{}".format(task, backend, model_type)


def safe_get(_object, key, alt):
    val = _object.get(key)
    if val is None or str(val) is None:
        return alt
    return val


def is_error(_object):
    """
    poor man's type checking, apparently you can not do `type(_object) == BackendError` when you move that class in the
    same file
    :param _object:
    :return:
    """
    if hasattr(_object, 'response_type'):
        return True
    return False


def serialize_experiment(exp):
    """
    serialize an Experiment object for consumption by swagger client
    :param exp: backends.backend.Experiment
    :return:
    """
    if is_error(exp):
        return abort(500, exp.message)
    train_events = [ServerResult(**r.__dict__) for r in exp.train_events]
    valid_events = [ServerResult(**r.__dict__) for r in exp.valid_events]
    test_events = [ServerResult(**r.__dict__) for r in exp.test_events]
    d = exp.__dict__
    d.update({'train_events': train_events})
    d.update({'valid_events': valid_events})
    d.update({'test_events': test_events})
    return ServerExperiment(**d)


def serialize_experiment_aggregate(agg_exps):
    """
    serialize an ExperimentAggregate object for consumption by swagger client
    :param agg_exps: backends.backend.ExperimentAggregate
    :return:
    """
    if is_error(agg_exps):
        return abort(500, agg_exps.message)
    results = []
    for agg_exp in agg_exps:
        train_events = [ServerAggregateResult(metric=r.metric,
                                              values=[AggregateResultValues(k, v) for k, v in r.values.items()])
                        for r in agg_exp.train_events]
        valid_events = [ServerAggregateResult(metric=r.metric,
                                              values=[AggregateResultValues(k, v) for k, v in r.values.items()])
                        for r in agg_exp.valid_events]
        test_events = [ServerAggregateResult(metric=r.metric,
                                             values=[AggregateResultValues(k, v) for k, v in r.values.items()])
                       for r in agg_exp.test_events]

        d = agg_exp.__dict__
        d.update({'train_events': train_events})
        d.update({'valid_events': valid_events})
        d.update({'test_events': test_events})
        results.append(ServerExperimentAggregate(**d))
    return results


def serialize_experiment_list(exps):
    """
    serialize a list of Experiment objects for consumption by swagger client
    :param exps:
    :return:
    """
    if is_error(exps):
        return abort(500, exps.message)
    results = []
    for exp in exps:
        if type(exp) == BackendError:
            return BackendResponse(**exp.__dict__)
        train_events = [ServerResult(**r.__dict__) for r in exp.train_events]
        valid_events = [ServerResult(**r.__dict__) for r in exp.valid_events]
        test_events = [ServerResult(**r.__dict__) for r in exp.test_events]
        d = exp.__dict__
        d.update({'train_events': train_events})
        d.update({'valid_events': valid_events})
        d.update({'test_events': test_events})
        results.append(ServerExperiment(**d))
    return results


def serialize_task_summary(task_summary):
    """
    serialize a TaskSummary object for consumption by swagger client
    :param task_summary:
    :return:
    """
    if is_error(task_summary):
        return abort(500, task_summary.message)
    return ServerTaskSummary(**task_summary.__dict__)


def serialize_task_summary_list(task_summaries):
    _task_summaries = []
    for task_summary in task_summaries:
        if not is_error(task_summary):  # should we abort if we cant get summary for a task in the database?
            _task_summaries.append(ServerTaskSummary(**task_summary.__dict__))
    return _task_summaries


def serialize_dict(config):
    """
    Serializes a dict, exists only to handle the error case
    :param config:
    :return:
    """
    if is_error(config):
        return abort(500, config.message)
    return config


def serialize_response(result):
    """
    serializes a Response object for swagger client consumption
    :param result:
    :return:
    """
    return ServerResponse(**result.__dict__)


def deserialize_result(result):
    return Result(
        metric=result.metric,
        value=result.value,
        tick_type=result.tick_type,
        tick=result.tick,
        phase=result.phase
    )


def pack_results_in_events(results):
    d = {}
    for result in results:
        if result.tick not in d:
            d[result.tick] = {result.metric: result.value,
                              'tick_type': result.tick_type,
                              'phase': result.phase,
                              'tick': result.tick
                              }
        else:
            d[result.tick].update({result.metric: result.value})
    return list(d.values())


def client_experiment_to_put_result_consumable(exp):
    train_events = pack_results_in_events(exp.train_events)
    valid_events = pack_results_in_events(exp.valid_events)
    test_events = pack_results_in_events(exp.test_events)
    config = exp.config
    task = exp.task
    extra_args = {
        'sha1': exp.sha1,
        'dataset': exp.dataset,
        'username': exp.username,
        'hostname': exp.hostname,
        'exp_date': exp.exp_date,
        'label': exp.label
    }
    put_result_consumable = namedtuple('put_result_consumable', ['task', 'config_obj', 'events_obj', 'extra_args'])
    return put_result_consumable(task=task, config_obj=json.loads(config),
                                 events_obj=train_events+valid_events+test_events,
                                 extra_args=extra_args)


def aggregate_results(resultset, reduction_dim, event_type, num_exps_per_reduction, prop_dict):
    # TODO: implement a trim method for ExperimentGroup
    # The resultset here is a query result from prop_dict as the filtering parameters, so these results should share
    # the parameters in prop_dict
    grouped_result = resultset.groupby(reduction_dim)
    aggregate_fns = {'min': np.min, 'max': np.max, 'avg': np.mean, 'std': np.std}
    return grouped_result.reduce(aggregate_fns=aggregate_fns, event_type=event_type, prop_dict=prop_dict)


def write_experiment(experiment, basedir):
    eid = str(experiment.eid)
    basedir = os.path.join(basedir, eid)
    os.makedirs(basedir)
    train_events = pack_results_in_events(experiment.train_events)
    valid_events = pack_results_in_events(experiment.valid_events)
    test_events = pack_results_in_events(experiment.test_events)
    json2log(train_events + valid_events + test_events, os.path.join(basedir, '{}-reporting.log'.format(eid)))
    config = json.loads(experiment.config) if type(experiment.config) is str else experiment.config
    write_config_file(config, os.path.join(basedir, '{}-config.yml'.format(eid)))
    d = experiment.__dict__
    [d.pop(event_type) for event_type in EVENT_TYPES]
    d.pop('config')
    write_config_file(d, os.path.join(basedir, '{}-meta.yml'.format(eid)))



from __future__ import print_function


EVENT_TYPES = {
    "train": "train_events", "Train": "train_events",
    "test": "test_events", "Test": "test_events",
    "valid": "valid_events", "Valid": "valid_events",
    "dev": "valid_events", "Dev": "valid_events"
}


class ExperimentRepo(object):

    def __init__(self):
        super(ExperimentRepo, self).__init__()

    def get_task_names(self):
        """Get the names of all tasks in the repository

        :return: A list of tasks
        """
        pass

    def config2dict(self, task, sha1):
        """Convert a configuration stored in the repository to a string

        :param task: (``str``) The task name
        :param sha1: (``str``) The sha1 of the configuration
        :return: (``dict``) The configuration
        """
        pass

    @staticmethod
    def create_repo(dbhost, dbport, user, passwd, dbtype='mongo'):
        """Create a MongoDB-backed repository

        :param dbtype: (``str``) The database type
        :param dbhost: (``str``) The host name
        :param dbport: (``str``) The port
        :param user: (``str``) The user
        :param passwd: (``str``)The password
        :return: A MongoDB/Sql-backed repository
        """
        if dbtype == 'mongo':
            from xpctl.backends.mongo.store import MongoRepo
            return MongoRepo(dbhost, int(dbport), user, passwd)
        else:
            from xpctl.backends.sql.store import SQLRepo
            return SQLRepo(type=dbtype, host=dbhost, port=dbport, user=user, passwd=passwd)

    def get_model_location(self, task, eid):
        """Get the physical location of the model specified by this experiment id

        :param task: (``str``) The task name
        :param eid: The identifier of the experiment
        :return: (``str``) The model location
        """
        pass

    def get_experiment_details(self, task, eid, event_type, metric):
        """Get detailed description for an experiment
        :param task: (``str``) The task name
        :param eid: The identifier of the experiment
        :param event_type: (List[``str``]) event types to listen for
        :param metric: (List[``str``]) The metric(s) to use
        :return: xpctl.backend.data.Experiment object
        """
        pass

    def get_results(self, task, param_dict, reduction_dim, metric, sort, numexp_reduction_dim, event_type):
        """Get results from the database
        :param task: (``str``) The taskname
        :param param_dict: (``dict``) The dict of parameters for query
        :param reduction_dim: (``str``) The property on which the results will be reduced
        :param metric: (``str``) The metric(s) to use
        :param sort: (``str``) The field to sort on
        :param numexp_reduction_dim: (``str``) number of results to show per group
        :param event_type: (``str``) event types to listen for
        :return: List[xpctl.backend.data.ExperimentAggregate]
        """
        pass

    def task_summary(self, task):
        """Summary for a task: What datasets were used? How many times each dataset was used?
        :param task: (``str``) Task name
        :return xpctl.backend.data.TaskSummary
        """
        pass

    def summary(self):
        """
        Summary for all tasks in the database
        :return: List[xpctl.backend.data.TaskSummary]
        """
        pass

    def find_experiments(self, task, prop, value):
        """
        find experiments by a property, eg: dataset
        :param task: (``str``) task name
        :param prop: (``str``) a property of an experiment, eg: dataset, label
        :param value: (``str``) value for the property
        :return: List[xpctl.backend.data.Experiment]
        """
        pass

    def update_prop(self, task, eid, prop, value):
        """
        Update a property(label, username etc) for an experiment
        :param task: (``str``) task name
        :param eid: The identifier for this record
        :param prop: (``str``) property to change
        :param value: (``str``) the value of the property
        :return: Union[xpctl.backend.data.Success, xpctl.backend.data.Error]
        """
        raise NotImplemented("Base ExperimentRepo events are immutable")

    def remove_experiment(self, task, eid):
        """Remove a record specified by this id
        :param task: (``str``) The task name for this record
        :param eid: The identifier for this record
        :return: Union[xpctl.backend.data.Success, xpctl.backend.data.Error]
        """
        raise NotImplemented("Base ExperimentRepo tasks are immutable")

    def put_result(self, task, experiment):
        """Put tan experiment in the database

        :param task: (``str``) The task name
        :param experiment: xpctl.backend.data.Experiment
        :return: Union[xpctl.backend.data.Success, xpctl.backend.data.Error]
        """
        pass

    def list_results(self, task, param_dict, user, metric, sort, event_type):
        """Get results from the database
        :param task: (``str``) The taskname
        :param param_dict: (``dict``) The dict of parameters for query
        :param user: List[``str``)] filter by users
        :param metric: (``str``) The metric(s) to use
        :param sort: (``str``) The field to sort on
        :param event_type: (``str``) event types to listen for
        :return: List[xpctl.backend.data.Experiment]
        """
        pass

    def dump(self, zipfile, task_eids):
        """
        dump the whole database. creates a zipfile which unzips into the following directory structure
        <xpctldump>
         - <task>
           - <id>
             - <id>-reporting.log
             - <id>-config.yml
             - <id>-meta.yml (any meta info such as label, username etc.)

        :param zipfile: zip file location for the dump. defaults to xpctldump-datetimestamp.zip
        :param task_eids: a dictionary of the form {'task': [eid1, eid2]}, if you want to dump specific files.
        :return: the path to the dumped file
        """
        pass

    def restore(self, dump):
        """
        restore a database from the dump. be careful: the experiment ids will change.
        :param dump: dump file location
        :return:
        """
        pass



#!/usr/bin/env python3

import connexion
import logging
import argparse
import os
from xpctl.backends.core import ExperimentRepo
from baseline.utils import read_config_file

from xpctl.xpserver import encoder


def get_logging_level(ll):
    ll = ll.lower()
    if ll == 'debug':
        return logging.DEBUG
    if ll == 'info':
        return logging.INFO
    return logging.WARNING


def main():
    parser = argparse.ArgumentParser(description='xpctl server')
    parser.add_argument('--ll', help='Log level', type=str, default='INFO')
    parser.add_argument('--backend', help='backend', type=str, default='mongo')
    parser.add_argument('--cred', help='credential for backend',
                        default=os.path.expanduser('~/xpctlcred-mongo-local.json'))
    parser.add_argument('--user', help='user for backend')
    parser.add_argument('--passwd', help='user for backend')
    parser.add_argument('--dbhost', help='host for backend')
    parser.add_argument('--dbport', help='port for backend')
    parser.add_argument('--port', help='port', default='5310')
    args = parser.parse_args()
    
    logging.basicConfig(level=get_logging_level(args.ll))
    app = connexion.App(__name__, specification_dir='./swagger/')
    if args.user is not None:
        d = {
            'user': args.user,
            'passwd': args.passwd,
            'dbhost': args.dbhost,
            'dbport': args.dbport
        }
    else:
        d = read_config_file(args.cred)
    d.update({'dbtype': args.backend})
    app.app.backend = ExperimentRepo().create_repo(**d)
    app.app.json_encoder = encoder.JSONEncoder
    app.add_api('swagger.yaml', arguments={'title': 'xpctl'})
    app.run(port=args.port)


if __name__ == '__main__':
    main()



import os
import re
import shutil
from setuptools import setup, find_packages
from xpctl import __version__


class About(object):
    NAME = 'xpctl'
    AUTHOR = 'mead-ml'
    VERSION = __version__
    EMAIL = "mead.baseline@gmail.com"
    URL = "https://www.github.com/{}/{}".format(AUTHOR, NAME)
    DOC_NAME = "docs/{}.md".format(NAME)
    DOC_URL = "{}/docs/".format(URL)


def read_doc(f_name, new_name=None):
    """
    Because our readme is outside of this dir we need to copy it in so
    that it is picked up by the install.
    """
    if new_name is None:
        new_name = f_name
    path = os.path.dirname(os.path.realpath(__file__))
    doc_loc = os.path.normpath(os.path.join(path, '..', f_name))
    new_loc = os.path.join(path, new_name)
    if os.path.isfile(doc_loc):
        shutil.copyfile(doc_loc, new_loc)
    descript = open(new_loc, 'r').read()
    return descript


def main():
    setup(
        name='mead-{}'.format(About.NAME),
        version=About.VERSION,
        description='Experiment Control and Tracking',
        long_description=read_doc(About.DOC_NAME, "README.md"),
        long_description_content_type="text/markdown",
        author=About.AUTHOR,
        author_email=About.EMAIL,
        license='Apache 2.0',
        url=About.URL,
        packages=find_packages(),
        install_requires=[
            'Click',
            'click-shell',
            'pandas',
            'xlsxwriter',
            'jsondiff',
            'pyyaml',
            'certifi>=14.05.14',
            'six>=1.10',
            'python_dateutil>=2.5.3',
            'setuptools>=21.0.0',
            'urllib3>=1.15.1',
        ],
        entry_points={
            'console_scripts': [
                'xpctl = xpctl.cli:cli'
            ],
        },
        extras_require={
            'test': [],
            'mongo': [
                'pymongo',
            ],
            'sql': [
                'sqlalchemy', 
                'psycopg2',
            ]
        },
        classifiers={
            'Development Status :: 2 - Pre-Alpha',
            'Environment :: Console',
            'Intended Audience :: Developers',
            'Intended Audience :: Science/Research',
            'License :: OSI Approved :: Apache Software License',
            'Natural Language :: English',
            'Operating System :: OS Independent',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6',
            'Topic :: Scientific/Engineering :: Artificial Intelligence',
        },
        keywords=['experiment control', 'tracking'],
    )


if __name__ == "__main__":
    main()



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from xpctl.xpserver.models.base_model_ import Model
from xpctl.xpserver import util


class AggregateResultValues(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, aggregate_fn=None, score=None):  # noqa: E501
        """AggregateResultValues - a model defined in Swagger

        :param aggregate_fn: The aggregate_fn of this AggregateResultValues.  # noqa: E501
        :type aggregate_fn: str
        :param score: The score of this AggregateResultValues.  # noqa: E501
        :type score: float
        """
        self.swagger_types = {
            'aggregate_fn': str,
            'score': float
        }

        self.attribute_map = {
            'aggregate_fn': 'aggregate_fn',
            'score': 'score'
        }

        self._aggregate_fn = aggregate_fn
        self._score = score

    @classmethod
    def from_dict(cls, dikt):
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The AggregateResult_values of this AggregateResultValues.  # noqa: E501
        :rtype: AggregateResultValues
        """
        return util.deserialize_model(dikt, cls)

    @property
    def aggregate_fn(self):
        """Gets the aggregate_fn of this AggregateResultValues.


        :return: The aggregate_fn of this AggregateResultValues.
        :rtype: str
        """
        return self._aggregate_fn

    @aggregate_fn.setter
    def aggregate_fn(self, aggregate_fn):
        """Sets the aggregate_fn of this AggregateResultValues.


        :param aggregate_fn: The aggregate_fn of this AggregateResultValues.
        :type aggregate_fn: str
        """

        self._aggregate_fn = aggregate_fn

    @property
    def score(self):
        """Gets the score of this AggregateResultValues.


        :return: The score of this AggregateResultValues.
        :rtype: float
        """
        return self._score

    @score.setter
    def score(self, score):
        """Sets the score of this AggregateResultValues.


        :param score: The score of this AggregateResultValues.
        :type score: float
        """

        self._score = score



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from xpctl.xpserver.models.base_model_ import Model
from xpctl.xpserver.models.aggregate_result_values import AggregateResultValues  # noqa: F401,E501
from xpctl.xpserver import util


class AggregateResult(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, metric=None, values=None):  # noqa: E501
        """AggregateResult - a model defined in Swagger

        :param metric: The metric of this AggregateResult.  # noqa: E501
        :type metric: str
        :param values: The values of this AggregateResult.  # noqa: E501
        :type values: List[AggregateResultValues]
        """
        self.swagger_types = {
            'metric': str,
            'values': List[AggregateResultValues]
        }

        self.attribute_map = {
            'metric': 'metric',
            'values': 'values'
        }

        self._metric = metric
        self._values = values

    @classmethod
    def from_dict(cls, dikt):
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The AggregateResult of this AggregateResult.  # noqa: E501
        :rtype: AggregateResult
        """
        return util.deserialize_model(dikt, cls)

    @property
    def metric(self):
        """Gets the metric of this AggregateResult.


        :return: The metric of this AggregateResult.
        :rtype: str
        """
        return self._metric

    @metric.setter
    def metric(self, metric):
        """Sets the metric of this AggregateResult.


        :param metric: The metric of this AggregateResult.
        :type metric: str
        """
        if metric is None:
            raise ValueError("Invalid value for `metric`, must not be `None`")  # noqa: E501

        self._metric = metric

    @property
    def values(self):
        """Gets the values of this AggregateResult.


        :return: The values of this AggregateResult.
        :rtype: List[AggregateResultValues]
        """
        return self._values

    @values.setter
    def values(self, values):
        """Sets the values of this AggregateResult.


        :param values: The values of this AggregateResult.
        :type values: List[AggregateResultValues]
        """
        if values is None:
            raise ValueError("Invalid value for `values`, must not be `None`")  # noqa: E501

        self._values = values
